# Generated by roxygen2: do not edit by hand

export(get_account_adjust_leverage_info)
export(get_account_balance)
export(get_account_bill_types)
export(get_account_bills)
export(get_account_bills_archive)
export(get_account_bills_history_archive)
export(get_account_collateral_assets)
export(get_account_config)
export(get_account_greeks)
export(get_account_instruments)
export(get_account_interest_accrued)
export(get_account_interest_limits)
export(get_account_interest_rate)
export(get_account_leverage_info)
export(get_account_max_avail_size)
export(get_account_max_loan)
export(get_account_max_size)
export(get_account_max_withdrawal)
export(get_account_mmp_config)
export(get_account_move_positions_history)
export(get_account_position_risk)
export(get_account_position_tiers)
export(get_account_positions)
export(get_account_positions_history)
export(get_account_precheck_set_delta_neutral)
export(get_account_risk_state)
export(get_account_set_account_switch_precheck)
export(get_account_spot_borrow_repay_history)
export(get_account_subaccount_balances)
export(get_account_subaccount_max_withdrawal)
export(get_account_subtypes)
export(get_account_trade_fee)
export(get_asset_asset_valuation)
export(get_asset_balances)
export(get_asset_bills)
export(get_asset_bills_history)
export(get_asset_convert_currencies)
export(get_asset_convert_currency_pair)
export(get_asset_convert_history)
export(get_asset_currencies)
export(get_asset_deposit_address)
export(get_asset_deposit_history)
export(get_asset_deposit_withdraw_status)
export(get_asset_exchange_list)
export(get_asset_non_tradable_assets)
export(get_asset_transfer_state)
export(get_asset_withdrawal_history)
export(get_copy_trade_config)
export(get_copy_trade_current_subpos)
export(get_copy_trade_historical_subpos)
export(get_copy_trade_instruments)
export(get_copy_trade_my_leaders)
export(get_copy_trade_profit_sharing_details)
export(get_copy_trade_public_config)
export(get_copy_trade_public_copy_traders)
export(get_copy_trade_public_current_subpositions)
export(get_copy_trade_public_lead_traders)
export(get_copy_trade_public_pnl)
export(get_copy_trade_public_preference_currency)
export(get_copy_trade_public_stats)
export(get_copy_trade_public_subpositions_history)
export(get_copy_trade_public_weekly_pnl)
export(get_copy_trade_settings)
export(get_copy_trade_total_profit_sharing)
export(get_copy_trade_total_unrealized_profit_sharing)
export(get_copy_trade_unrealized_profit_sharing_details)
export(get_market_block_ticker)
export(get_market_block_tickers)
export(get_market_books)
export(get_market_books_rpi)
export(get_market_candles)
export(get_market_exchange_rate)
export(get_market_history_candles)
export(get_market_history_index_candles)
export(get_market_history_mark_price_candles)
export(get_market_history_trades)
export(get_market_index_candles)
export(get_market_index_components)
export(get_market_index_tickers)
export(get_market_mark_price_candles)
export(get_market_option_instrument_family_trades)
export(get_market_platform_24_volume)
export(get_market_ticker)
export(get_market_tickers)
export(get_market_trades)
export(get_public_block_trades)
export(get_public_convert_contract_coin)
export(get_public_delivery_exercise_history)
export(get_public_discount_rate_interest_free_quota)
export(get_public_economic_calendar)
export(get_public_estimated_price)
export(get_public_estimated_settlement_info)
export(get_public_event_contract_events)
export(get_public_event_contract_markets)
export(get_public_event_contract_series)
export(get_public_funding_rate)
export(get_public_funding_rate_history)
export(get_public_instrument_tick_bands)
export(get_public_instruments)
export(get_public_insurance_fund)
export(get_public_interest_rate_loan_quota)
export(get_public_mark_price)
export(get_public_mm_instrument_types)
export(get_public_open_interest)
export(get_public_opt_summary)
export(get_public_option_trades)
export(get_public_position_tiers)
export(get_public_premium_history)
export(get_public_price_limit)
export(get_public_settlement_history)
export(get_public_time)
export(get_public_underlying)
export(get_trade_account_rate_limit)
export(get_trade_easy_convert_currency_list)
export(get_trade_easy_convert_history)
export(get_trade_fills)
export(get_trade_fills_history)
export(get_trade_one_click_repay_currency_list)
export(get_trade_one_click_repay_currency_list_v2)
export(get_trade_one_click_repay_history)
export(get_trade_one_click_repay_history_v2)
export(get_trade_order)
export(get_trade_order_algo)
export(get_trade_orders_algo_history)
export(get_trade_orders_algo_pending)
export(get_trade_orders_history)
export(get_trade_orders_history_7d)
export(get_trade_orders_history_archive)
export(get_trade_orders_pending)
export(get_user_glp_historical_performance)
export(get_user_glp_today_performance)
export(get_var_label)
export(post_account_account_level_switch_preset)
export(post_account_mmp_config)
export(post_account_mmp_reset)
export(post_account_move_positions)
export(post_account_position_margin_balance)
export(post_account_set_account_level)
export(post_account_set_auto_loan)
export(post_account_set_auto_repay)
export(post_account_set_collateral_assets)
export(post_account_set_fee_type)
export(post_account_set_greeks)
export(post_account_set_leverage)
export(post_account_set_position_mode)
export(post_account_spot_manual_borrow_repay)
export(post_asset_cancel_withdrawal)
export(post_asset_convert_estimate_quote)
export(post_asset_convert_trade)
export(post_asset_transfer)
export(post_asset_withdrawal)
export(post_trade_amend_algos)
export(post_trade_amend_batch_orders)
export(post_trade_amend_order)
export(post_trade_batch_orders)
export(post_trade_cancel_algos)
export(post_trade_cancel_all_after)
export(post_trade_cancel_batch_orders)
export(post_trade_cancel_order)
export(post_trade_close_position)
export(post_trade_mass_cancel)
export(post_trade_order)
export(post_trade_order_algo)
export(post_trade_order_precheck)
export(set_okxr_options)
importFrom(data.table,as.data.table)
importFrom(httr,content)
importFrom(rlang,"%||%")
