% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_auth.R
\name{.execute_post_action}
\alias{.execute_post_action}
\title{Execute a POST request to OKX}
\usage{
.execute_post_action(api_path, body_list, config)
}
\arguments{
\item{api_path}{API path (e.g., "/api/v5/trade/order").}

\item{body_list}{List to be converted to JSON for the request body.}

\item{config}{List with API credentials.}
}
\value{
An `httr` response object, or `NULL` if the request fails.
}
\description{
Perform a signed POST call to the OKX API.
}
