% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_interest_rate}
\alias{get_account_interest_rate}
\title{Get account borrowing interest rates}
\usage{
get_account_interest_rate(
  ccy = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Currency filter, e.g. `"BTC"`.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `ccy` and `interestRate`.
}
\description{
Retrieve the current leveraged currency borrowing market interest rate for
one currency or for all eligible currencies.
}
