% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_block_trades}
\alias{get_public_block_trades}
\title{Get public block trades}
\usage{
get_public_block_trades(
  inst_id,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with block trade fields such as price, size, trade
  side, volatility, and timestamps.
}
\description{
Retrieve recent single-leg public block trades for an instrument.
}
