% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_order_precheck}
\alias{post_trade_order_precheck}
\title{Precheck a Trade Order}
\usage{
post_trade_order_precheck(
  inst_id,
  td_mode,
  side,
  ord_type,
  sz,
  ccy = NULL,
  cl_ord_id = NULL,
  tag = NULL,
  pos_side = NULL,
  px = NULL,
  reduce_only = NULL,
  tgt_ccy = NULL,
  attach_algo_ords = NULL,
  speed_bump = NULL,
  outcome = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Instrument ID.}

\item{td_mode}{Trade mode.}

\item{side}{Order side.}

\item{ord_type}{Order type.}

\item{sz}{Order size.}

\item{ccy}{Optional margin currency.}

\item{cl_ord_id}{Optional client order ID.}

\item{tag}{Optional order tag.}

\item{pos_side}{Optional position side.}

\item{px}{Optional order price.}

\item{reduce_only}{Optional logical reduce-only flag.}

\item{tgt_ccy}{Optional target currency mode.}

\item{attach_algo_ords}{Optional attached TP/SL list.}

\item{speed_bump}{Optional event-contract speed bump.}

\item{outcome}{Optional event-contract outcome.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with projected account metrics after the precheck.
}
\description{
Submit an order precheck request without placing the order.
}
