% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_unrealized_profit_sharing_details}
\alias{get_copy_trade_unrealized_profit_sharing_details}
\title{Get unrealized profit sharing details}
\usage{
get_copy_trade_unrealized_profit_sharing_details(
  inst_type = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with unrealized profit sharing rows.
}
\description{
Retrieve unrealized profit sharing detail rows.
}
