% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_precheck_set_delta_neutral}
\alias{get_account_precheck_set_delta_neutral}
\title{Precheck delta-neutral strategy switch}
\usage{
get_account_precheck_set_delta_neutral(
  stgy_type,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{stgy_type}{Character or numeric. Strategy type. `"0"` for general or
`"1"` for delta neutral.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with JSON-encoded unmatched information.
}
\description{
Retrieve unmatched information that blocks switching into the requested
strategy type.
}
