% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_amend_order}
\alias{post_trade_amend_order}
\title{Amend a Trade Order}
\usage{
post_trade_amend_order(
  inst_id,
  ord_id = NULL,
  cl_ord_id = NULL,
  req_id = NULL,
  new_sz = NULL,
  new_px = NULL,
  cxl_on_fail = NULL,
  new_px_usd = NULL,
  new_px_vol = NULL,
  px_amend_type = NULL,
  attach_algo_ords = NULL,
  speed_bump = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Instrument ID.}

\item{ord_id}{Order ID. Optional if `cl_ord_id` is supplied.}

\item{cl_ord_id}{Client order ID. Optional if `ord_id` is supplied.}

\item{req_id}{Optional client amendment request ID.}

\item{new_sz}{Optional new total order size.}

\item{new_px}{Optional new price.}

\item{cxl_on_fail}{Optional logical. Whether to cancel the order if the
amendment fails.}

\item{new_px_usd}{Optional new option order USD price.}

\item{new_px_vol}{Optional new option order implied volatility price.}

\item{px_amend_type}{Optional price amendment mode.}

\item{attach_algo_ords}{Optional attached TP/SL amendment list.}

\item{speed_bump}{Optional event-contract speed bump.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` describing the amendment request result.
}
\description{
Submit an amendment request for an incomplete order.
}
