% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_easy_convert_currency_list}
\alias{get_trade_easy_convert_currency_list}
\title{Get easy convert currency list}
\usage{
get_trade_easy_convert_currency_list(
  source = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{source}{Character or `NULL`. Source account type filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps.}
}
\value{
A `data.frame` with source and target currency metadata.
}
\description{
Retrieve currencies available for easy convert.
}
