% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_time}
\alias{get_public_time}
\title{Get OKX system time}
\usage{
get_public_time(config = NULL, tz = .okx_default_tz)
}
\arguments{
\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with system time.
}
\description{
Retrieve OKX system time.
}
