% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_historical_subpos}
\alias{get_copy_trade_historical_subpos}
\title{Get historical copy trading subpositions}
\usage{
get_copy_trade_historical_subpos(
  inst_type = NULL,
  inst_id = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{inst_id}{Character or `NULL`. Instrument ID filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with fields like `instId` and `uniqueCode`.
}
\description{
Retrieve your historical copy trading subpositions.
}
\details{
Wraps `/api/v5/copytrading/subpositions-history`. Returns one row per
closed or historical subposition.
}
\note{
Since okxr 0.1.2
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
hist <- get_copy_trade_historical_subpos(config = cfg)
head(hist)
}

}
\seealso{
[get_copy_trade_current_subpos()]
}
