% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_account_level_switch_preset}
\alias{post_account_account_level_switch_preset}
\title{Preset Account Level Switch}
\usage{
post_account_account_level_switch_preset(
  acct_lv,
  lever = NULL,
  risk_offset_type = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{acct_lv}{Target account level.}

\item{lever}{Optional leverage preset.}

\item{risk_offset_type}{Optional deprecated risk offset type field.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` describing the stored preset values.
}
\description{
Preset required values before switching account mode.
}
