% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_position_tiers}
\alias{get_public_position_tiers}
\title{Get public position tiers}
\usage{
get_public_position_tiers(
  inst_type,
  td_mode,
  inst_family = NULL,
  inst_id = NULL,
  ccy = NULL,
  tier = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type.}

\item{td_mode}{Character. Trade mode.}

\item{inst_family}{Character or `NULL`. Instrument family.}

\item{inst_id}{Character or `NULL`. Instrument ID(s).}

\item{ccy}{Character or `NULL`. Margin currency.}

\item{tier}{Character or `NULL`. Tier filter.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with public position tier information.
}
\description{
Retrieve public tier, margin, and maximum leverage information.
}
