% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_fee_type}
\alias{post_account_set_fee_type}
\title{Set Account Fee Type}
\usage{
post_account_set_fee_type(fee_type, tz = .okx_default_tz, config)
}
\arguments{
\item{fee_type}{Fee type string, typically `"0"` or `"1"`.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied fee type.
}
\description{
Configure the fee charging mode for spot trading.
}
