% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_mass_cancel}
\alias{post_trade_mass_cancel}
\title{Mass Cancel MMP Orders}
\usage{
post_trade_mass_cancel(
  inst_type,
  inst_family,
  lock_interval = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Instrument type. Currently `OPTION`.}

\item{inst_family}{Instrument family.}

\item{lock_interval}{Optional lock interval in milliseconds.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with the request result.
}
\description{
Cancel all MMP pending orders for an options instrument family.
}
