% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_platform_24_volume}
\alias{get_market_platform_24_volume}
\title{Get platform 24-hour volume}
\usage{
get_market_platform_24_volume(
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with `volUsd`, `volCny`, and `ts`.
}
\description{
Retrieve total platform order-book trading volume over the last 24 hours.
}
