% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_auto_loan}
\alias{post_account_set_auto_loan}
\title{Set Account Auto Loan}
\usage{
post_account_set_auto_loan(
  auto_loan = TRUE,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{auto_loan}{Logical. Whether auto loan should be enabled.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied auto-loan setting.
}
\description{
Enable or disable automatic borrowing.
}
