% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_account_rate_limit}
\alias{get_trade_account_rate_limit}
\title{Get trade account rate limit}
\usage{
get_trade_account_rate_limit(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with rate-limit metrics such as
  `accRateLimit`, `fillRatio`, `mainFillRatio`, `nextAccRateLimit`, and `ts`.
}
\description{
Retrieve account rate limit information related to new and amended order
requests.
}
\details{
Wraps `/api/v5/trade/account-rate-limit`. Returns one row containing the
current account rate limit and fill-ratio metrics used by OKX.
}
