% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_asset_valuation}
\alias{get_asset_asset_valuation}
\title{Get asset valuation}
\usage{
get_asset_asset_valuation(
  ccy = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Valuation currency filter.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with valuation summary rows.
}
\description{
Retrieve total account valuation for funding assets.
}
