% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_max_loan}
\alias{get_account_max_loan}
\title{Get account maximum loan}
\usage{
get_account_max_loan(
  mgn_mode,
  inst_id = NULL,
  ccy = NULL,
  mgn_ccy = NULL,
  trade_quote_ccy = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{mgn_mode}{Character. Margin mode: `"cross"` or `"isolated"`.}

\item{inst_id}{Character or `NULL`. Instrument ID(s).}

\item{ccy}{Character or `NULL`. Currency.}

\item{mgn_ccy}{Character or `NULL`. Margin currency.}

\item{trade_quote_ccy}{Character or `NULL`. Quote currency for trading.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `instId`, `mgnMode`, `mgnCcy`, `maxLoan`, `ccy`,
  and `side`.
}
\description{
Retrieve the maximum loan for manual borrow or margin borrowing scenarios.
}
