% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_config}
\alias{get_copy_trade_config}
\title{Get copy trading config}
\usage{
get_copy_trade_config(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with top-level copy trading configuration fields.
  Nested `details` are returned as a JSON string column.
}
\description{
Retrieve your account-level copy trading configuration.
}
