% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_mmp_reset}
\alias{post_account_mmp_reset}
\title{Reset MMP Status}
\usage{
post_account_mmp_reset(
  inst_family,
  inst_type = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{inst_family}{Instrument family.}

\item{inst_type}{Optional instrument type. Defaults to `"OPTION"` on OKX.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` with the request result.
}
\description{
Reset market maker protection status for an instrument family.
}
