% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_orders_algo_pending}
\alias{get_trade_orders_algo_pending}
\title{Get pending algo orders}
\usage{
get_trade_orders_algo_pending(
  ord_type,
  algo_id = NULL,
  inst_type = NULL,
  inst_id = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ord_type}{Character. Algo order type filter.}

\item{algo_id}{Character or `NULL`. Algo order ID.}

\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{inst_id}{Character or `NULL`. Instrument ID filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of results to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps.}
}
\value{
A `data.frame` with pending algo orders.
}
\description{
Retrieve untriggered algo orders.
}
