% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_convert_currency_pair}
\alias{get_asset_convert_currency_pair}
\title{Get convert currency pair}
\usage{
get_asset_convert_currency_pair(
  from_ccy,
  to_ccy,
  convert_mode = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{from_ccy}{Character. Currency to convert from.}

\item{to_ccy}{Character. Currency to convert to.}

\item{convert_mode}{Character or `NULL`. Convert mode selector.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with currency-pair convert metadata.
}
\description{
Retrieve convert metadata for a currency pair.
}
