% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_mmp_config}
\alias{get_account_mmp_config}
\title{Get account MMP configuration}
\usage{
get_account_mmp_config(
  inst_family = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_family}{Character or `NULL`. Instrument family filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with MMP configuration fields.
}
\description{
Retrieve option-market-maker-protection configuration for one or more
instrument families.
}
