% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_transfer_state}
\alias{get_asset_transfer_state}
\title{Get asset transfer state}
\usage{
get_asset_transfer_state(
  trans_id = NULL,
  client_id = NULL,
  type = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{trans_id}{Character or `NULL`. Transfer ID.}

\item{client_id}{Character or `NULL`. Client-supplied transfer ID.}

\item{type}{Character or `NULL`. Transfer type.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with transfer state rows.
}
\description{
Retrieve the state of a funding transfer.
}
